@@ -10,6 +10,8 @@ from urllib import request
 import zipfile
 import tarfile
 import argparse
+import filecmp
+import difflib
 
 
 # logger
@@ -323,8 +325,6 @@ class HomeMan:
             else:
                 raise HomeManException(f'cannot copy: {usr_path}')
             data['mode'] = bin(os.stat(man_path).st_mode)
-            ctime_ns = os.stat(man_path).st_ctime
-            os.utime(usr_path, (ctime_ns, ctime_ns))
             self.__mark_changed()
             logger.info('sync reversely: %s', data['homeman'])
         else:
@@ -346,8 +346,6 @@ class HomeMan:
             else:
                 raise HomeManException(f'cannot copy: {man_path}')
             os.chmod(usr_path, int(data['mode'], 2))
-            ctime_ns = os.stat(usr_path).st_ctime
-            os.utime(man_path, (ctime_ns, ctime_ns))
             logger.info('sync: %s', data['homeman'])
 
     def backup(self, route: str, force: bool):
@@ -417,7 +415,7 @@ class HomeMan:
             os.removedirs(usr_path)
         logger.info('restore: %s', route)
 
-    def check(self, route: str):
+    def check(self, route: str, show_diff: bool):
         logger.debug('========< check >========')
         data = self.get_data(route)
         if data is None:
@@ -425,11 +423,11 @@ class HomeMan:
             return
         for homeman_data in self.__scan_data(data):
             try:
-                self.__check(homeman_data)
+                self.__check(homeman_data, show_diff)
             except HomeManException as e:
                 logger.warning(e)
     
-    def __check(self, data: Dict[str, str]):
+    def __check(self, data: Dict[str, str], show_diff: bool):
         route = data['homeman']
         usr_path = self.usr_home + route
         usr_path_bak = usr_path + '.homeman.bak'
@@ -440,25 +438,57 @@ class HomeMan:
             man_path = self.man_home + route
             if not os.path.exists(man_path):
                 logger.warning('not exists inside: %s', route)
+            elif not os.path.exists(usr_path):
+                logger.warning('not exists outside: %s', route)
             else:
-                def get_ctime(path: str) -> int:
-                    if os.path.isfile(path):
-                        return os.stat(path).st_ctime_ns
-                    elif os.path.isdir(path):
-                        ctime = os.stat(path).st_ctime_ns
-                        for name in os.listdir(path):
-                            new_ctime = get_ctime(path + os.path.sep + name)
-                            if new_ctime > ctime:
-                                ctime = new_ctime
-                        return ctime
-                    else:
-                        raise HomeManException(f'path error: {path}')
-                man_time = get_ctime(man_path)
-                usr_time = get_ctime(usr_path)
-                if man_time != usr_time:
-                    logger.warning('not sync: %s', route)
-                else:
+                diff_files = []
+                if os.path.isfile(usr_path) and os.path.isfile(man_path):
+                    if not filecmp.cmp(usr_path, man_path):
+                        diff_files.append(route)
+                elif os.path.isdir(usr_path) and os.path.isdir(man_path):
+                    def collect_diff_files(dcmp, dfs: List[str]):
+                        for name in dcmp.diff_files:
+                            dfs.append(dcmp.left[len(self.usr_home):] + os.path.sep + name)
+                        for sub_dcmp in dcmp.subdirs.values():
+                            collect_diff_files(sub_dcmp, dfs)
+                    collect_diff_files(filecmp.dircmp(usr_path, man_path), diff_files)
+                if len(diff_files) == 0:
                     logger.info('sync: %s', route)
+                else:
+                    logger.warning('no sync: %s', route)
+                    if show_diff:
+                        differ = difflib.Differ()
+                        for f in diff_files:
+                            print(f'[{f}]')
+                            usr_f = self.usr_home + f
+                            man_f = self.man_home + f
+                            if not os.path.exists(usr_f):
+                                print('not exists outside')
+                            elif not os.path.exists(man_f):
+                                print('not exists inside')
+                            elif os.path.isdir(usr_f):
+                                print('is dir outside')
+                            elif os.path.isdir(man_f):
+                                print('is dir inside')
+                            else:
+                                with open(usr_f, 'r') as uf, open(man_f, 'r') as mf:
+                                    res = ['------ start ------\n']
+                                    res.extend(list(differ.compare(uf.readlines(), mf.readlines())))
+                                    res.append('------ end ------\n')
+                                    common_counter = 0
+                                    last_line = '\n'
+                                    for line in res:
+                                        if line.startswith('  '):
+                                            common_counter += 1
+                                            last_line = line
+                                        else:
+                                            if common_counter != 0:
+                                                if common_counter == 1:
+                                                    print(last_line, end='')
+                                                else:
+                                                    print(f'------ {common_counter} common lines ------')
+                                            print(line, end='')
+                                            common_counter = 0
 
 
 def main():
@@ -572,6 +602,10 @@ def main():
         'path', type=str,
         help='the location of resource'
     )
+    command_check.add_argument(
+        '-v', '--show', action='store_true',
+        help='show diff'
+    )
 
     args = homeman_parse.parse_args()
 
@@ -607,7 +641,7 @@ def main():
             elif args.command == 'restore':
                 hm.restore(route)
             elif args.command == 'check':
-                hm.check(route)
+                hm.check(route, args.show)
     except HomeManException as e:
         logger.error(e)
         return 1
